@@ -1,6 +1,5 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995-1998, 2000-2003, 2006, 2008, 2015-2016 Free Software
-   Foundation, Inc.
+   Copyright (C) 1995-2025 Free Software Foundation, Inc.
 
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
@@ -15,16 +14,18 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifndef _WRITE_PO_H
 #define _WRITE_PO_H
 
-#include "ostream.h"
-#include "message.h"
-
 #include <stdbool.h>
 
+#include <textstyle.h>
+
+#include "attribute.h"
+#include "message.h"
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -39,14 +40,16 @@ enum filepos_comment_type
   };
 
 /* These functions are used to output a #, flags line.  */
-extern const char *
+extern char *
        make_format_description_string (enum is_format is_format,
-                                       const char *lang, bool debug);
+                                       const char *lang, bool debug)
+       ATTRIBUTE_MALLOC;
 extern bool
        significant_format_p (enum is_format is_format);
 
 extern char *
-       make_range_description_string (struct argument_range range);
+       make_range_description_string (struct argument_range range)
+       ATTRIBUTE_MALLOC;
 
 /* These functions output parts of a message, as comments.  */
 extern void
@@ -55,7 +58,8 @@ extern void
        message_print_comment_dot (const message_ty *mp, ostream_t stream);
 extern void
        message_print_comment_filepos (const message_ty *mp, ostream_t stream,
-                                      bool uniforum, size_t page_width);
+                                      const char *charset, bool uniforum,
+                                      size_t page_width);
 extern void
        message_print_comment_flags (const message_ty *mp, ostream_t stream,
                                     bool debug);
@@ -69,6 +73,8 @@ extern void
        message_print_style_uniforum (void);
 extern void
        message_print_style_escape (bool flag);
+extern void
+       message_print_style_comment (bool flag);
 extern void
        message_print_style_filepos (enum filepos_comment_type type);
 
@@ -77,7 +83,7 @@ extern bool handle_filepos_comment_option (const char *option);
 
 
 /* Describes a PO file in .po syntax.  */
-extern DLL_VARIABLE const struct catalog_output_format output_format_po;
+extern LIBGETTEXTSRC_DLL_VARIABLE const struct catalog_output_format output_format_po;
 
 
 #ifdef __cplusplus
